import getpass
import json
import os
import unittest

import requests
import requests_oauthlib

CPI_ROOT_URL = os.getenv(
    'CPI_ROOT_URL', 'https://search.apps.staging.ubuntu.com')
SSO_ROOT_URL = os.getenv(
    'SSO_ROOT_URL', 'https://login.staging.ubuntu.com')

TEST_USER_EMAIL = os.getenv('TEST_USER_EMAIL', None)
if TEST_USER_EMAIL is None:
    TEST_USER_EMAIL = input('Test user email: ')
TEST_USER_PASSWORD = os.getenv('TEST_USER_PASSWORD', None)
if TEST_USER_PASSWORD is None:
    TEST_USER_PASSWORD = getpass.getpass('Test user password: ')

CLICK_PACKAGE_NAME = 'demo37.ricardokirkner'
SNAP_PACKAGE_NAME = 'hello-world'
SNAP_PACKAGE_CHANNEL = 'stable'
SNAP_PACKAGE_REVISION = 6
SNAP_PACKAGE_SNAP_ID = 'JtwEnisYi8Mmk51vNLZPSOwSOFLwGdhs'
PRIVATE_SNAP_PACKAGE_NAME = 'ricardokirkner-test1'
PRIVATE_SNAP_PACKAGE_SNAP_ID = 'qrwoXeYOXAZla2ga3XtBSogV1vAUKqZh'
PRIVATE_SNAP_PACKAGE_CHANNEL = 'stable'
PRIVATE_SNAP_PACKAGE_REVISION = 1


def get_oauth_token_data(email, password):
    url = '{}/api/v2/tokens/oauth'.format(SSO_ROOT_URL)
    data = {
        'email': email,
        'password': password,
        'token_name': 'store-acceptance-tests',
    }
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }
    response = requests.post(url, data=json.dumps(data), headers=headers)
    assert response.ok, 'Failed to get oauth token: %r' % response.content

    data = response.json()
    return {
        'client_key': data['consumer_key'],
        'client_secret': data['consumer_secret'],
        'resource_owner_key': data['token_key'],
        'resource_owner_secret': data['token_secret'],
    }


def get_oauth_client():
    credentials = get_oauth_token_data(TEST_USER_EMAIL, TEST_USER_PASSWORD)
    client = requests_oauthlib.OAuth1Session(**credentials)
    client.headers.update({
        'Content-Type': 'application/json',
        'Cache-Control': 'no-cache',
    })
    return client


class APITestCase(unittest.TestCase):

    def assert_success(self, response, content_type='application/hal+json'):
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers['Content-Type'],
                         content_type)

    def assert_not_found(self, response, message=None, new_style_error=False):
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.headers['Content-Type'],
                         'application/hal+json')
        expected = {
            'result': 'error',
            'errors': [message or 'Not found'],
        }
        if new_style_error:
            expected['error_list'] = [
                {'code': 'resource-not-found', 'message': message},
            ]
        self.assertEqual(response.json(), expected)

    def assert_bad_request(self, response, message, code='',
                           new_style_error=False):
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.headers['Content-Type'],
                         'application/hal+json')
        expected = {
            'result': 'error',
            'errors': [message],
        }
        if new_style_error:
            expected['error_list'] = [
                {'code': code, 'message': message},
            ]

        self.assertEqual(response.json(), expected)

    def assert_unauthorized(self, response):
        self.assertEqual(response.status_code, 401)
        self.assertEqual(response.headers['Content-Type'],
                         'application/hal+json')
        self.assertIn('WWW-Authenticate', response.headers)
