import requests

from .helpers import (
    CPI_ROOT_URL,
    APITestCase,
)


def get_departments(include_subdepartments=None):
    url = '{}/api/v1/departments'.format(CPI_ROOT_URL)
    if include_subdepartments is True:
        url += '?include_subdepartments=true'
    elif include_subdepartments is False:
        url += '?include_subdepartments=false'
    response = requests.get(url)
    return response


def get_department(slug):
    url = '{}/api/v1/departments/{}'.format(CPI_ROOT_URL, slug)
    response = requests.get(url)
    return response


class DepartmentsTestCase(APITestCase):

    def assert_departments(self, response):
        body = response.json()
        # assert generic embedded data structure
        self.assertIn('_embedded', body)
        self.assertIn('clickindex:department', body['_embedded'])
        # assert there is at least one result
        self.assertGreater(len(body['_embedded']['clickindex:department']), 0)
        # assert result data structure
        item = body['_embedded']['clickindex:department'][0]
        self._assert_department_data(item)

    def assert_department(self, response, slug):
        body = response.json()
        # assert result data structure
        self._assert_department_data(body, slug=slug)

    def _assert_department_data(self, data, slug=None):
        self.assertIn('has_children', data)
        self.assertIn('name', data)
        self.assertIn('slug', data)
        if slug is not None:
            self.assertEqual(data['slug'], slug)

    def find_subdepartment(self, response, department, slug):
        body = response.json()
        for item in body.get('_embedded', {}).get('clickindex:department', []):
            department_match = item.get('slug') == department
            if department_match and item.get('has_children', False):
                for subitem in item.get('_embedded', {}).get(
                        'clickindex:department', []):
                    if subitem.get('slug') == slug:
                        return subitem

    def test_get_departments(self):
        response = get_departments()
        self.assert_success(response)
        self.assert_departments(response)

    def test_get_department(self):
        response = get_department(slug='games')
        self.assert_success(response)
        self.assert_department(response, slug='games')

    def test_include_subdepartments_by_default(self):
        response = get_departments()
        self.assert_success(response)
        department = self.find_subdepartment(
            response, 'games', slug='fps')
        self.assertIsNotNone(department)
        self._assert_department_data(department, slug='fps')

    def test_exclude_subdepartments(self):
        response = get_departments(include_subdepartments=False)
        self.assert_success(response)
        department = self.find_subdepartment(response, 'games', slug='fps')
        self.assertIsNone(department)
