import json

import requests

from .helpers import (
    CLICK_PACKAGE_NAME,
    CPI_ROOT_URL,
    SNAP_PACKAGE_SNAP_ID,
    APITestCase,
)


def get_click_metadata(names=None, method='get'):
    url = '{}/api/v1/click-metadata'.format(CPI_ROOT_URL)
    if method == 'get':
        if names is not None:
            url += '?' + '&'.join('name=' + name for name in names)
        response = requests.get(url)
    elif method == 'post':
        data = {}
        if names is not None:
            data['name'] = names
        response = requests.post(url, data=json.dumps(data),
                                 headers={'Content-Type': 'application/json'})
    else:
        raise ValueError('invalid method: %s' % method)
    return response


class ClickMetadataTestCase(APITestCase):

    def assert_click_metadata(self, response, name):
        body = response.json()
        self.assertEqual(len(body), 1)
        # assert result data structure
        self._assert_package_data(body[0], name=name)

    def _assert_package_data(self, data, name):
        self.assertIn('binary_filesize', data)
        self.assertIn('changelog', data)
        self.assertIn('channel', data)
        self.assertIn('content', data)
        self.assertIn('department', data)
        self.assertIn('download_sha512', data)
        self.assertIn('download_url', data)
        self.assertIn('icon_url', data)
        self.assertIn('name', data)
        self.assertIn('origin', data)
        self.assertIn('revision', data)
        self.assertIn('status', data)
        self.assertIn('summary', data)
        self.assertIn('title', data)
        self.assertIn('version', data)
        self.assertEqual(data['name'], name)

    def test_get_click_metadata(self):
        response = get_click_metadata(names=[CLICK_PACKAGE_NAME])
        self.assert_success(response, content_type='application/json')
        self.assert_click_metadata(response, name=CLICK_PACKAGE_NAME)

    def test_get_click_metadata_no_packages(self):
        response = get_click_metadata()
        self.assert_bad_request(
            response,
            message='Package names must be specified in parameter "name".')

    def test_get_click_metadata_via_post(self):
        response = get_click_metadata(
            names=[CLICK_PACKAGE_NAME], method='post')
        self.assert_success(response, content_type='application/json')
        self.assert_click_metadata(response, name=CLICK_PACKAGE_NAME)

    def test_get_click_metadata_no_packages_via_post(self):
        response = get_click_metadata(method='post')
        self.assert_bad_request(
            response,
            message='Package names must be specified in parameter "name".')

    def test_get_click_metadata_bad_request_via_post(self):
        url = '{}/api/v1/click-metadata'.format(CPI_ROOT_URL)
        response = requests.post(url, data='invalid data',
                                 headers={'Content-Type': 'application/json'})
        self.assert_bad_request(
            response, message='No JSON object could be decoded')


class MetadataTestCase(APITestCase):
    endpoint = '/api/v1/metadata'

    def get_metadata(self, snaps=None, fields=None):
        url = '{}{}'.format(CPI_ROOT_URL, self.endpoint)
        data = {}
        if snaps is not None:
            data['snaps'] = snaps
        if fields is not None:
            data['fields'] = fields
        response = requests.post(
            url, data=json.dumps(data),
            headers={'Content-Type': 'application/json'})
        return response

    def assert_metadata(self, response, fields=None, snap_id=None):
        body = response.json()
        # assert generic embedded data structure
        self.assertIn('_embedded', body)
        self.assertIn('clickindex:package', body['_embedded'])
        # assert there is at least one result
        self.assertGreater(len(body['_embedded']['clickindex:package']), 0)
        # assert result data structure
        item = body['_embedded']['clickindex:package'][0]
        self._assert_package_data(item, fields=fields, snap_id=snap_id)

    def _assert_package_data(self, data, fields=None, snap_id=None):
        if fields is None:
            fields = [
                'anon_download_url',
                'binary_filesize',
                'changelog',
                'channel',
                'content',
                'department',
                'download_sha512',
                'download_url',
                'icon_url',
                'name',
                'origin',
                'package_name',
                'plugs',
                'revision',
                'slots',
                'snap_id',
                'status',
                'summary',
                'title',
                'version',
            ]
        expected = fields + ['_links']
        self.assertEqual(set(data.keys()), set(expected))
        if snap_id is not None:
            self.assertEqual(data['snap_id'], snap_id)

    def assert_no_results(self, response):
        body = response.json()
        self._assert_package_data(body, fields=[])

    def test_get_metadata(self):
        response = self.get_metadata(
            snaps=[{'snap_id': SNAP_PACKAGE_SNAP_ID}])
        self.assert_success(response)
        self.assert_metadata(response, snap_id=SNAP_PACKAGE_SNAP_ID)

    def test_get_metadata_no_snaps(self):
        response = self.get_metadata()
        self.assert_bad_request(
            response,
            code='missing-field',
            message='Packages must be specified in parameter "snaps".',
            new_style_error=True)

    def test_get_metadata_missing_snap_id(self):
        response = self.get_metadata(snaps=[{}])
        self.assert_bad_request(
            response, code='missing-field',
            message='Missing snap_id for snap.',
            new_style_error=True)

    def test_get_metadata_filter_fields(self):
        fields = ['channel', 'revision', 'snap_id']
        response = self.get_metadata(
            snaps=[{'snap_id': SNAP_PACKAGE_SNAP_ID}],
            fields=fields)
        self.assert_success(response)
        self.assert_metadata(
            response, fields=fields, snap_id=SNAP_PACKAGE_SNAP_ID)

    def test_get_metadata_invalid_package(self):
        response = self.get_metadata(
            snaps=[{'snap_id': 'invalid'}])
        self.assert_success(response)
        self.assert_no_results(response)
