import requests

from .helpers import (
    CPI_ROOT_URL,
    PRIVATE_SNAP_PACKAGE_CHANNEL,
    PRIVATE_SNAP_PACKAGE_NAME,
    PRIVATE_SNAP_PACKAGE_REVISION,
    PRIVATE_SNAP_PACKAGE_SNAP_ID,
    SNAP_PACKAGE_CHANNEL,
    SNAP_PACKAGE_NAME,
    SNAP_PACKAGE_REVISION,
    SSO_ROOT_URL,
    TEST_USER_EMAIL,
    TEST_USER_PASSWORD,
    APITestCase,
    get_oauth_client,
)


def get_package(package_id, channel=None, revision=None, auth=False):
    url = '{}/api/v1/package/{}'.format(CPI_ROOT_URL, package_id)
    if channel is not None:
        url += '/{}'.format(channel)
    elif revision is not None:
        url += '/{}'.format(revision)
    if auth:
        # use client as contextmanager to ensure ssl socket is closed
        # afterwards
        with get_oauth_client() as client:
            response = client.get(url)
    else:
        response = requests.get(url)
    return response


class PackageBaseTestCase(APITestCase):

    def assert_package(self, response, snap_id=None, name=None,
                       channel=None, revision=None, exclude_fields=None):
        body = response.json()
        fields = [
            'alias',
            'allow_unauthenticated',
            'anon_download_url',
            'architecture',
            'binary_filesize',
            'blacklist_country_codes',
            'changelog',
            'channel',
            'channels',
            'click_framework',
            'click_version',
            'company_name',
            'confinement',
            'content',
            'date_published',
            'department',
            'description',
            'developer_id',
            'developer_name',
            'download_sha512',
            'download_sha3_384',
            'download_url',
            'epoch',
            'framework',
            'icon_url',
            'icon_urls',
            'id',
            'is_published',
            'keywords',
            'last_updated',
            'license',
            'name',
            'origin',
            'package_name',
            'plugs',
            'price',
            'prices',
            'promoted',
            'publisher',
            'ratings_average',
            'release',
            'revision',
            'screenshot_url',
            'screenshot_urls',
            'slots',
            'snap_id',
            'status',
            'summary',
            'support_url',
            'terms_of_service',
            'title',
            'version',
            'video_urls',
            'website',
            'whitelist_country_codes',
        ]
        # assert result data structure
        expected = set(fields + ['_links'])
        if exclude_fields:
            expected = expected - set(exclude_fields)
        self.assertEqual(set(body.keys()), expected)
        if snap_id is not None:
            self.assertEqual(body['snap_id'], snap_id)
        if name is not None:
            self.assertEqual(body['package_name'], name)
        if channel is not None:
            self.assertEqual(body['channel'], channel)
            self.assertIn(channel, body['channels'])
        if revision is not None:
            self.assertEqual(body['revision'], revision)


class PackageTestCase(PackageBaseTestCase):

    def test_get_package(self):
        response = get_package(SNAP_PACKAGE_NAME)
        self.assert_success(response)
        self.assert_package(response, name=SNAP_PACKAGE_NAME)

    def test_get_private_package_when_not_authorized(self):
        response = get_package(PRIVATE_SNAP_PACKAGE_SNAP_ID)
        self.assert_not_found(response, message='No such package')

    def test_get_private_package_when_authorized(self):
        response = get_package(PRIVATE_SNAP_PACKAGE_SNAP_ID, auth=True)
        self.assert_success(response)
        self.assert_package(
            response, snap_id=PRIVATE_SNAP_PACKAGE_SNAP_ID,
            name=PRIVATE_SNAP_PACKAGE_NAME)

    def test_get_package_not_found(self):
        response = get_package('missing')
        self.assert_not_found(response, message='No such package')


class ChannelPackageTestCase(PackageBaseTestCase):

    def test_get_channel_package(self):
        response = get_package(SNAP_PACKAGE_NAME, channel=SNAP_PACKAGE_CHANNEL)
        self.assert_success(response)
        self.assert_package(response, name=SNAP_PACKAGE_NAME,
                            channel=SNAP_PACKAGE_CHANNEL)

    def test_get_private_channel_package_when_not_authorized(self):
        response = get_package(PRIVATE_SNAP_PACKAGE_SNAP_ID,
                               channel=PRIVATE_SNAP_PACKAGE_CHANNEL)
        self.assert_not_found(response, message='No such package')

    def test_get_private_channel_package_when_authorized(self):
        response = get_package(PRIVATE_SNAP_PACKAGE_SNAP_ID,
                               channel=PRIVATE_SNAP_PACKAGE_CHANNEL, auth=True)
        self.assert_success(response)
        self.assert_package(
            response, snap_id=PRIVATE_SNAP_PACKAGE_SNAP_ID,
            name=PRIVATE_SNAP_PACKAGE_NAME, channel=PRIVATE_SNAP_PACKAGE_CHANNEL)

    def test_get_channel_package_not_found(self):
        response = get_package('missing', channel='stable')
        self.assert_not_found(response, message='No such package')

    def test_get_channel_package_wrong_channel(self):
        response = get_package(SNAP_PACKAGE_NAME, channel='missing')
        self.assert_not_found(response, message='No such package')


class PackageRevisionTestCase(PackageBaseTestCase):

    def test_get_package_revision(self):
        response = get_package(SNAP_PACKAGE_NAME, revision=SNAP_PACKAGE_REVISION)
        self.assert_success(response)
        self.assert_package(
            response, name=SNAP_PACKAGE_NAME, revision=SNAP_PACKAGE_REVISION,
            exclude_fields=['channel'])

    def test_get_private_package_revision_when_not_authorized(self):
        response = get_package(PRIVATE_SNAP_PACKAGE_SNAP_ID,
                               revision=PRIVATE_SNAP_PACKAGE_REVISION)
        self.assert_not_found(response, message='No such package')

    def test_get_private_package_when_authorized(self):
        response = get_package(PRIVATE_SNAP_PACKAGE_SNAP_ID,
                               revision=PRIVATE_SNAP_PACKAGE_REVISION, auth=True)
        self.assert_success(response)
        self.assert_package(
            response, snap_id=PRIVATE_SNAP_PACKAGE_SNAP_ID,
            name=PRIVATE_SNAP_PACKAGE_NAME, exclude_fields=['channel'])

    def test_get_package_revision_not_found(self):
        response = get_package(SNAP_PACKAGE_NAME, revision=-1)
        self.assert_not_found(response, message='No such package')
