from urllib.parse import urlencode

import requests

from .helpers import (
    CPI_ROOT_URL,
    PRIVATE_SNAP_PACKAGE_NAME,
    APITestCase,
    get_oauth_client,
)


class SearchTestCase(APITestCase):
    endpoint = '/api/v1/search'

    def setUp(self):
        super(SearchTestCase, self).setUp()
        self.name = 'hello'

    def search(self, name=None, fields=None, auth=False):
        query_data = {}
        if name is not None:
            query_data['name'] = name

        query = {}
        if query_data:
            query['q'] = '&'.join(
                '{}:{}'.format(key, value)
                for key, value in query_data.items())
        if fields:
            query['fields'] = fields

        return self._search(query=query, auth=auth)

    def _search(self, query=None, auth=False):
        url = '{}{}'.format(CPI_ROOT_URL, self.endpoint)
        if query:
            qs = urlencode(query)
            url += '?' + qs
        if auth:
            with get_oauth_client() as client:
                response = client.get(url)
        else:
            response = requests.get(url)
        return response

    def assert_results(self, response, name=None):
        body = response.json()
        # assert generic embedded data structure
        self.assertIn('_embedded', body)
        self.assertIn('clickindex:package', body['_embedded'])
        # assert there is at least one result
        self.assertGreater(len(body['_embedded']['clickindex:package']), 0)
        # assert result data structure
        item = body['_embedded']['clickindex:package'][0]
        if name is not None:
            self.assertIn(name, item['name'])

    def assert_no_results(self, response, name):
        body = response.json()
        embedded = body.get('_embedded', {})
        results = embedded.get('clickindex:package', [])
        for item in results:
            self.assertNotIn(name, item['name'])

    def assert_recommendations(self, response):
        body = response.json()
        self.assertIn('_embedded', body)
        self.assertIn('clickindex:recommendation', body['_embedded'])
        # assert there is at least one result
        self.assertGreater(
            len(body['_embedded']['clickindex:recommendation']), 0)

    def test_search_by_name(self):
        response = self.search(name=self.name)
        self.assert_success(response)
        self.assert_results(response, self.name)

    def test_search_all(self):
        response = self.search()
        self.assert_success(response)
        self.assert_results(response)

    def test_search_private_not_authorized(self):
        response = self.search(name=PRIVATE_SNAP_PACKAGE_NAME)
        self.assert_success(response)
        self.assert_no_results(response, name=PRIVATE_SNAP_PACKAGE_NAME)

    def test_search_private(self):
        response = self.search(name=PRIVATE_SNAP_PACKAGE_NAME, auth=True)
        self.assert_success(response)
        self.assert_results(response, name=PRIVATE_SNAP_PACKAGE_NAME)
