import requests

from .helpers import (
    CPI_ROOT_URL,
    SNAP_PACKAGE_NAME,
    PRIVATE_SNAP_PACKAGE_NAME,
    APITestCase,
    get_oauth_client,
)
from .test_metadata import MetadataTestCase
from .test_search import SearchTestCase


def get_snap_detail(name, series='16', auth=False):
    url = '{}/api/v1/snaps/details/{}'.format(CPI_ROOT_URL, name)
    headers = {}
    if series:
        headers['X-Ubuntu-Series'] = series
    if auth:
        with get_oauth_client() as client:
            response = client.get(url, headers=headers)
    else:
        response = requests.get(url, headers=headers)
    return response


class SnapDetailTestCase(APITestCase):

    def assert_snap_detail(self, response, name):
        body = response.json()
        fields = [
            'alias',
            'allow_unauthenticated',
            'anon_download_url',
            'architecture',
            'binary_filesize',
            'blacklist_country_codes',
            'changelog',
            'channel',
            'channels',
            'click_framework',
            'click_version',
            'company_name',
            'confinement',
            'content',
            'date_published',
            'department',
            'description',
            'developer_id',
            'developer_name',
            'download_sha512',
            'download_url',
            'epoch',
            'framework',
            'icon_url',
            'icon_urls',
            'id',
            'is_published',
            'keywords',
            'last_updated',
            'license',
            'name',
            'origin',
            'package_name',
            'plugs',
            'price',
            'prices',
            'promoted',
            'publisher',
            'ratings_average',
            'release',
            'revision',
            'screenshot_url',
            'screenshot_urls',
            'slots',
            'snap_id',
            'status',
            'summary',
            'support_url',
            'terms_of_service',
            'title',
            'version',
            'video_urls',
            'website',
            'whitelist_country_codes',
        ]
        for field in fields:
            self.assertIn(field, body)
        self.assertEqual(body['package_name'], name)

    def test_get_snap_detail(self):
        response = get_snap_detail(SNAP_PACKAGE_NAME)
        self.assert_success(response)
        self.assert_snap_detail(response, SNAP_PACKAGE_NAME)

    def test_get_snap_detail_for_private_package(self):
        response = get_snap_detail(PRIVATE_SNAP_PACKAGE_NAME, auth=True)
        self.assert_success(response)
        self.assert_snap_detail(response, PRIVATE_SNAP_PACKAGE_NAME)

    def test_get_snap_detail_without_series(self):
        response = get_snap_detail(SNAP_PACKAGE_NAME, series=None)
        self.assert_bad_request(
            response, code='missing-header',
            message='X-Ubuntu-Series header is required.',
            new_style_error=True)

    def test_get_snap_detail_unauthorized(self):
        response = get_snap_detail(PRIVATE_SNAP_PACKAGE_NAME, auth=False)
        self.assert_not_found(response, message='No such package',
                              new_style_error=True)

    def test_get_snap_detail_not_found(self):
        response = get_snap_detail('missing')
        self.assert_not_found(response, message='No such package',
                              new_style_error=True)


class SnapMetadataTestCase(MetadataTestCase):
    endpoint = '/api/v1/snaps/metadata'


class SnapSearchTestCase(SearchTestCase):
    endpoint = '/api/v1/snaps/search'

    def search(self, name=None, fields=None, private=False, auth=False):
        query = {}
        if name is not None:
            query['name'] = name
        if private:
            query['private'] = 'true'
        if fields is not None:
            query['fields'] = fields
        return super(SnapSearchTestCase, self)._search(
            query=query, auth=auth)

    def test_search_private_not_authorized(self):
        response = self.search(name=PRIVATE_SNAP_PACKAGE_NAME, private=True)
        self.assert_unauthorized(response)

    def test_search_private(self):
        response = self.search(name=PRIVATE_SNAP_PACKAGE_NAME, private=True,
                               auth=True)
        self.assert_success(response)
        self.assert_results(response, name=PRIVATE_SNAP_PACKAGE_NAME)
