import getpass
import glob
import json
import os
import shutil
import subprocess
import tempfile
import unittest
import urllib.parse
import uuid

import requests
from pymacaroons import Macaroon


TEST_USER_EMAIL = os.getenv('TEST_USER_EMAIL', None)
if TEST_USER_EMAIL is None:
    TEST_USER_EMAIL = input('Test user email: ')
TEST_USER_PASSWORD = os.getenv('TEST_USER_PASSWORD', None)
if TEST_USER_PASSWORD is None:
    TEST_USER_PASSWORD = getpass.getpass('Test user password: ')
TEST_USER_NAMESPACE = os.getenv('TEST_USER_NAMESPACE', None)
if TEST_USER_NAMESPACE is None:
    TEST_USER = input('Test user namespace: ')

CUD_ROOT_URL = os.getenv(
    'CUD_ROOT_URL', 'https://upload.apps.staging.ubuntu.com')
SCA_ROOT_URL = os.getenv(
    'SCA_ROOT_URL', 'https://myapps.developer.staging.ubuntu.com')
SSO_ROOT_URL = os.getenv(
    'SSO_ROOT_URL', 'https://login.staging.ubuntu.com')
SSO_SERVICE_LOCATION = urllib.parse.urlparse(SSO_ROOT_URL).netloc


def authenticate_with_macaroon(permissions=None, packages=None):
    if permissions is None:
        permissions = ['package_upload']
    root = get_root_macaroon(permissions=permissions, packages=packages)
    caveat_id = get_caveat_id(root)
    discharge = get_discharge_macaroon(caveat_id)
    return bind_macaroon(root, discharge)


def assert_response_ok(response):
    msg = 'Response was: [%r] %r' % (
        response.status_code,
        response.content)
    assert response.ok, msg


def get_root_macaroon(permissions, packages=None):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }
    data = {
        'permissions': permissions,
    }
    if packages is not None:
        data['packages'] = packages
    response = requests.post(
        '{}/dev/api/acl/'.format(SCA_ROOT_URL),
        data=json.dumps(data),
        headers=headers)
    assert_response_ok(response)
    return response.json()['macaroon']


def get_caveat_id(root):
    mac = Macaroon.deserialize(root)
    caveat, = [
        c for c in mac.third_party_caveats()
        if c.location == SSO_SERVICE_LOCATION
    ]
    return caveat.caveat_id


def get_discharge_macaroon(caveat_id):
    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }
    data = {
        'email': TEST_USER_EMAIL,
        'password': TEST_USER_PASSWORD,
        'caveat_id': caveat_id,
    }
    response = requests.post(
        '{}/api/v2/tokens/discharge'.format(SSO_ROOT_URL),
        data=json.dumps(data),
        headers=headers)
    assert_response_ok(response)
    return response.json()['discharge_macaroon']


def bind_macaroon(root, discharge):
    root_mac = Macaroon.deserialize(root)
    discharge_mac = Macaroon.deserialize(discharge)
    bound = root_mac.prepare_for_request(discharge_mac)
    return 'Macaroon root={}, discharge={}'.format(
        root, bound.serialize())


def create_snap(name, version=None):
    """Create a test snap from a template.

    :param name: The name of the snap package
    :param version: An optional version for the created snap

    :returns: The path where the binary snap file has been created,
              the folder where the binary snap file has been created in
              and the version used for the snap file.
    """
    if version is None:
        # Change to a random version. The maximum size is 32 chars.
        version = str(uuid.uuid4().int)[:32]

    template = '''
name: {name}
version: {version}
architectures: ['all']
confinement: devmode
summary: Summary of the most simple test snap
description: Description of the most simple test snap

parts:
    part1:
      plugin: nil
'''
    snap_dir = tempfile.mkdtemp()
    yaml_path = os.path.join(snap_dir, 'snapcraft.yaml')
    with open(yaml_path, 'w') as f:
        f.write(template.format(name=name, version=version))

    proc = subprocess.Popen(
        ['snapcraft'], cwd=snap_dir,
        stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    proc.communicate()
    assert proc.returncode == 0

    snap_files = glob.glob(
        os.path.join(snap_dir, '{}_{}_*.snap'.format(name, version)))
    assert len(snap_files) == 1
    snap_path = snap_files[0]
    return snap_path, snap_dir, version


def upload_snap(name):
    url = '{}/unscanned-upload/'.format(CUD_ROOT_URL)
    snap_path, snap_dir, version = create_snap(name)
    try:
        with open(snap_path, 'rb') as f:
            files = {'binary': f}
            response = requests.post(url, files=files)
    finally:
        shutil.rmtree(snap_dir)
    if response.ok:
        body = response.json()
        return body.get('upload_id'), version


class APITestCase(unittest.TestCase):

    # Do not ever truncate diffs so we don't ever miss response body changes
    maxDiff = None

    def assert_success(self, response, content_type='application/json'):
        self.assertTrue(response.ok, 'Response was: [{}] {}'.format(
            response.status_code, response.content))
        self.assertEqual(response.headers['Content-Type'], content_type)

    def assert_error(self, response, content_type='application/json'):
        self.assertFalse(response.ok, 'Response was: [{}] {}'.format(
            response.status_code, response.content))
        self.assertEqual(response.headers['Content-Type'], content_type)

    def assert_require_auth(self, response):
        self.assertEqual(response.status_code, 401)

    def assert_forbidden(self, response, new_style=False):
        self.assertEqual(response.status_code, 403)
        body = response.json()
        expected = {
            "status": 403,
            "permission": "package_upload",
            "type": "devportal:v1:macaroon-permission-required",
            "detail": "Permission is required: package_upload",
            "title": "Macaroon missing required permission.",
        }
        if new_style:
            expected["error_list"] = [{
                "code": "macaroon-permission-required",
                "message": "Permission is required: package_upload",
            }]
        self.assertEqual(body, expected)

    def assert_not_found(self, response):
        self.assertEqual(response.status_code, 404)
        body = response.json()
        expected = {
            "status": 404,
        }
        self.assertEqual(body, expected)
