from __future__ import unicode_literals

import json
import time
import uuid

import requests  # fades

from .helpers import (
    SCA_ROOT_URL,
    TEST_USER_NAMESPACE,
    APITestCase,
    authenticate_with_macaroon,
)


def register_name(name, series=None, is_private=False, dry_run=False,
                  auth=None, retry=True):
    headers = {
        'Content-Type': 'application/json',
    }
    if auth is not None:
        headers['Authorization'] = auth
    data = {'snap_name': name}
    if series is not None:
        data['series'] = series
    if is_private is not None:
        data['is_private'] = is_private
    url = '{}/dev/api/register-name/'.format(SCA_ROOT_URL)
    if dry_run:
        url += '?dry_run'
    response = requests.post(url, data=json.dumps(data), headers=headers)

    if retry and response.status_code == 429:
        wait = int(response.headers['Retry-After'])
        time.sleep(wait)
        response = requests.post(url, data=json.dumps(data), headers=headers)

    return response


class RegisterNameTestCase(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(RegisterNameTestCase, cls).setUpClass()

        cls.auth = authenticate_with_macaroon()

    def setUp(self):
        super(RegisterNameTestCase, self).setUp()

        # use same format for package names as snapcraft integration tests
        self.package_name = 'delete-me-{}'.format(str(uuid.uuid4().int)[:32])

    def assert_snap_id(self, response):
        self.assertIn('snap_id', response.json())

    def assert_no_snap_id(self, response):
        snap_id = response.json().get('snap_id')
        self.assertIsNone(snap_id)

    def assert_already_claimed(self, response, name):
        self.assertEqual(response.status_code, 409)
        body = response.json()
        expected = {
            'status': 409,
            'code': 'already_claimed',
            'title': 'Name already registered.',
            'detail': "You already registered a claim for '%s'." % name,
            'type': 'devportal:v1:name-already-registered',
            'error_list': [{
                'code': 'already_claimed',
                'message': "You already registered a claim for '%s'." % name
            }]
        }
        self.assertEqual(body, expected)

    def assert_already_owned(self, response, name):
        self.assertEqual(response.status_code, 409)
        body = response.json()
        expected = {
            'status': 409,
            'code': 'already_owned',
            'title': 'Name already registered.',
            'detail': "You already own '%s'." % name,
            'type': 'devportal:v1:name-already-registered',
            'error_list': [{
                'code': 'already_owned',
                'message': "You already own '%s'." % name
            }]
        }
        self.assertEqual(body, expected)

    def assert_already_registered(self, response, name):
        self.assertEqual(response.status_code, 409)
        body = response.json()
        register_name_url = (
            '{}/dev/click-apps/register-name/'
            '?series=16&name={}'.format(SCA_ROOT_URL, name))
        expected = {
            'status': 409,
            'code': 'already_registered',
            'title': 'Name already registered.',
            'detail': "'%s' is already registered." % name,
            'suggested_snap_name': '{}-{}'.format(TEST_USER_NAMESPACE, name),
            'register_name_url': register_name_url,
            'type': 'devportal:v1:name-already-registered',
            'error_list': [{
                'code': 'already_registered',
                'message': "'%s' is already registered." % name
            }]
        }
        self.assertEqual(body, expected)

    def assert_reserved_name(self, response, name):
        self.assertEqual(response.status_code, 409)
        body = response.json()
        register_name_url = (
            '{}/dev/click-apps/register-name/'
            '?series=16&name={}'.format(SCA_ROOT_URL, name))
        expected = {
            'status': 409,
            'code': 'reserved_name',
            'title': 'Name is reserved.',
            'detail': "'%s' is a reserved name." % name,
            'suggested_snap_name': '{}-{}'.format(TEST_USER_NAMESPACE, name),
            'register_name_url': register_name_url,
            'type': 'devportal:v1:name-reserved',
            'error_list': [{
                'code': 'reserved_name',
                'message': "'%s' is a reserved name." % name
            }]
        }
        self.assertEqual(body, expected)

    def assert_retry_after(self, response):
        self.assertEqual(response.status_code, 429)
        self.assertIn('Retry-After', response.headers)
        retry_after = response.headers['Retry-After']
        body = response.json()
        expected = {
            'status': 429,
            'code': 'register_window',
            'title': 'You must wait before next name registration.',
            'detail': 'You must wait 1\xa0minute before the next snap-name '
                    'registration for 16.',
            'retry_after': int(retry_after),
            'type': u'devportal:v1:name-window-wait',
            'error_list': [{
                'code': 'register_window',
                'message': 'You must wait 1\xa0minute before the next snap-name '
                        'registration for 16.',
            }]
        }
        self.assertEqual(body, expected)

    def assert_invalid_request(self, response):
        self.assertEqual(response.status_code, 400)
        body = response.json()
        expected = {
            'invalid_params': [
                {'reason': 'Select a valid choice. That choice is not one of '
                           'the available choices.',
                 'code': 'invalid_choice',
                 'name': u'series'}
            ],
            'status': 400,
            'type': 'devportal:v1:request-invalid',
            'detail': 'Submitted data is not valid.',
            'title': 'Invalid request.',
            'error_list': [{
                'message': 'Select a valid choice. That choice is not one of the available choices.',
                'code': 'invalid_choice',
                'extra': {'name': 'series'}}]
        }
        self.assertEqual(body, expected)

    def test_register_name_successful(self):
        response = register_name(self.package_name, auth=self.auth)
        self.assert_success(response)
        self.assert_snap_id(response)

    def test_register_name_already_registered(self):
        snap_name = 'test-already-registered-snap-name'
        response = register_name(snap_name, auth=self.auth)
        self.assert_error(response)
        self.assert_already_registered(response, snap_name)

    def test_register_name_already_claimed(self):
        snap_name = 'test-already-claimed-snap-name'
        response = register_name(snap_name, auth=self.auth)
        self.assert_error(response)
        self.assert_already_claimed(response, snap_name)

    def test_register_name_already_owned(self):
        register_name(self.package_name, auth=self.auth)
        response = register_name(self.package_name, auth=self.auth)
        self.assert_error(response)
        self.assert_already_owned(response, self.package_name)

    def test_register_name_reserved_name(self):
        name = '00reserved-name-for-acceptance-tests'
        response = register_name(name, auth=self.auth)
        self.assert_error(response)
        self.assert_reserved_name(response, name)

    def test_register_name_too_fast(self):
        register_name(self.package_name, auth=self.auth)
        response = register_name(
            self.package_name, auth=self.auth, retry=False)
        self.assert_error(response)
        self.assert_retry_after(response)

    def test_register_name_for_series(self):
        response = register_name(self.package_name, series='16',
                                 auth=self.auth)
        self.assert_success(response)
        self.assert_snap_id(response)

    def test_register_name_for_invalid_series(self):
        response = register_name(self.package_name, series='invalid',
                                 auth=self.auth)
        self.assert_error(response)
        self.assert_invalid_request(response)

    def test_register_private_name(self):
        response = register_name(self.package_name, is_private=True,
                                 auth=self.auth)
        self.assert_success(response)
        self.assert_snap_id(response)

    def test_check_but_not_register(self):
        response = register_name(self.package_name, dry_run=True,
                                 auth=self.auth)
        self.assert_success(response)
        self.assert_no_snap_id(response)

    def test_require_authentication(self):
        response = register_name(self.package_name)
        self.assert_require_auth(response)

    def test_require_package_upload_permission(self):
        auth = authenticate_with_macaroon(permissions=['package_access'])
        response = register_name(self.package_name, auth=auth)
        self.assert_error(response)
        self.assert_forbidden(response, new_style=True)
