from __future__ import unicode_literals

import json
import os

import requests  # fades
import stripe # fades

from .helpers import (
    SCA_ROOT_URL,
    APITestCase,
    authenticate_with_macaroon,
)


def create_customer(stripe_token, auth=None):
    headers = {
        'Content-Type': 'application/json',
    }
    if auth is not None:
        headers['Authorization'] = auth
    data = {}
    if stripe_token is not None:
        data['stripe_token'] = stripe_token
    url = '{}/purchases/v1/customers'.format(SCA_ROOT_URL)
    response = requests.post(url, data=json.dumps(data), headers=headers)
    return response


def get_stripe_token():
    stripe.api_key = os.getenv('STRIPE_PUBLISHABLE_KEY', None)
    token = stripe.Token.create(card={
        'number': '4111111111111111',
        'cvc': '1234',
        'exp_year': 2032,
        'exp_month': 12,
    })
    return token.id


def get_customer(auth=None):
    headers = {
        'Content-Type': 'application/json',
        'Cache-Control': 'no-cache',
    }
    if auth is not None:
        headers['Authorization'] = auth
    url = '{}/purchases/v1/customers/me'.format(SCA_ROOT_URL)
    response = requests.get(url, headers=headers)
    return response


class CreateCustomerTestCase(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(CreateCustomerTestCase, cls).setUpClass()

        cls.auth = authenticate_with_macaroon()

    def assert_invalid_request(self, response, message=None, extra=None):
        self.assertEqual(response.status_code, 400)
        body = response.json()
        if message is None:
            message = "The 'stripe_token' field is required."
        if extra is None:
            extra = {'field': 'stripe_token'}
        self.assertEqual(body, {
            'error_list': [
                {'code': 'invalid-field',
                 'message': message,
                 'extra': extra},
            ]
        })

    def assert_customer(self, response):
        self.assertEqual(response.status_code, 200)
        body = response.json()
        self.assertIn('accepted_tos_date', body)
        self.assertIn('latest_tos_date', body)
        self.assertIn('latest_tos_accepted', body)

    def test_require_authentication(self):
        response = create_customer(stripe_token='token')
        self.assert_require_auth(response)

    def test_create_customer_missing_token(self):
        response = create_customer(stripe_token=None, auth=self.auth)
        self.assert_invalid_request(response)

    def test_create_customer_invalid_token(self):
        response = create_customer(stripe_token='', auth=self.auth)
        message = "The 'stripe_token' field can not be empty."
        self.assert_invalid_request(response, message)

    def test_create_customer_bad_token(self):
        response = create_customer(stripe_token='invalid', auth=self.auth)
        message = "'invalid' is not a valid stripe_token."
        extra = {'field': 'stripe_token', 'value': 'invalid'}
        self.assert_invalid_request(response, message, extra)

    def test_create_customer_success(self):
        token = get_stripe_token()
        response = create_customer(stripe_token=token, auth=self.auth)
        self.assert_customer(response)


class GetCustomerTestCase(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(GetCustomerTestCase, cls).setUpClass()

        cls.auth = authenticate_with_macaroon()

    def assert_customer(self, response):
        data = response.json()
        self.assertIn('latest_tos_date', data)
        self.assertIn('accepted_tos_date', data)
        self.assertIn('latest_tos_accepted', data)

    def test_require_authentication(self):
        response = get_customer()
        self.assert_require_auth(response)

    def test_get_customer_successful(self):
        response = get_customer(auth=self.auth)
        self.assert_success(response)
        self.assert_customer(response)

    def test_get_customer_bad_auth(self):
        response = get_customer(auth='invalid')
        self.assert_require_auth(response)
