from __future__ import unicode_literals

import json

import requests  # fades

from .helpers import (
    SCA_ROOT_URL,
    APITestCase,
    authenticate_with_macaroon,
)
from .test_snap_purchases_customers import (
    create_customer,
    get_stripe_token,
)


SNAP_PACKAGE_SNAP_ID = 'A0KkM9O8Aoht03uQqKwGB7JEZAbTPwnu'


def create_order(snap_id, currency=None, amount=None, auth=None):
    headers = {
        'Content-Type': 'application/json',
    }
    if auth is not None:
        headers['Authorization'] = auth
    data = {}
    if snap_id is not None:
        data['snap_id'] = snap_id
    if currency is not None:
        data['currency'] = currency
    if amount is not None:
        data['amount'] = amount
    url = '{}/purchases/v1/orders'.format(SCA_ROOT_URL)
    response = requests.post(url, data=json.dumps(data), headers=headers)
    return response


def get_orders(auth=None):
    headers = {
        'Content-Type': 'application/json',
    }
    if auth is not None:
        headers['Authorization'] = auth
    url = '{}/purchases/v1/orders'.format(SCA_ROOT_URL)
    response = requests.get(url, headers=headers)
    return response


class OrdersTestCase(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(OrdersTestCase, cls).setUpClass()

        cls.auth = authenticate_with_macaroon(permissions=['package_purchase'])

    def assert_order_data(self, order):
        self.assertEqual(order['snap_id'], SNAP_PACKAGE_SNAP_ID)
        self.assertEqual(order['state'], 'Complete')
        self.assertEqual(order['currency'], 'USD')
        self.assertIn('amount', order)
        self.assertIsNotNone(order['amount'])


class CreateOrderTestCase(OrdersTestCase):

    def assert_invalid_request(self, response, message=None, extra=None):
        self.assertEqual(response.status_code, 400)
        body = response.json()
        if message is None:
            message = "The 'snap_id' field is required."
        if extra is None:
            extra = {'field': 'snap_id'}
        self.assertEqual(body, {
            'error_list': [
                {'code': 'invalid-field',
                 'message': message,
                 'extra': extra}
            ]
        })

    def assert_order(self, response):
        self.assertEqual(response.status_code, 200)
        body = response.json()
        self.assert_order_data(body)

    def test_require_authentication(self):
        response = create_order(snap_id='snap-id')
        self.assert_require_auth(response)

    def test_create_order_missing_snap_id(self):
        response = create_order(snap_id=None, auth=self.auth)
        self.assert_invalid_request(response)

    def test_create_order_invalid_currency(self):
        response = create_order(snap_id='snap-id', currency='EUR',
                                auth=self.auth)
        message = "'EUR' is not a valid currency."
        extra = {'field': 'currency', 'value': 'EUR'}
        self.assert_invalid_request(response, message, extra)

    def test_create_order_success(self):
        token = get_stripe_token()
        response = create_customer(stripe_token=token, auth=self.auth)
        assert response.status_code == 200
        response = create_order(snap_id=SNAP_PACKAGE_SNAP_ID, auth=self.auth)
        self.assert_order(response)


class GetOrdersTestCase(OrdersTestCase):

    def assert_orders(self, response):
        self.assertEqual(response.status_code, 200)
        body = response.json()
        self.assertIn('orders', body)
        orders = body['orders']
        if orders:
            order = body['orders'][-1]
            self.assert_order_data(order)

    def test_require_authentication(self):
        response = get_orders()
        self.assert_require_auth(response)

    def test_get_orders(self):
        response = get_orders(auth=self.auth)
        self.assert_orders(response)
