import json
import os
import uuid

import requests

from .helpers import (
    CUD_ROOT_URL,
    SCA_ROOT_URL,
    APITestCase,
    authenticate_with_macaroon,
    create_snap,
    upload_snap,
)

from .test_register_name import register_name


def push_snap(name=None, updown_id=None, series=None, auth=None):
    headers = {
        'Content-Type': 'application/json',
    }
    if auth is not None:
        headers['Authorization'] = auth

    data = {}
    if name is not None:
        data['name'] = name
    if updown_id is not None:
        data['updown_id'] = updown_id
    if series is not None:
        data['series'] = series

    url = '{}/dev/api/snap-push/'.format(SCA_ROOT_URL)
    response = requests.post(url, data=json.dumps(data), headers=headers)
    return response


class SnapPushTestCase(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(SnapPushTestCase, cls).setUpClass()

        cls.auth = authenticate_with_macaroon()

    def setUp(self):
        super(SnapPushTestCase, self).setUp()

        # use same format for package names as snapcraft integration tests
        self.package_name = 'delete-me-{}'.format(str(uuid.uuid4().int)[:32])
        response = register_name(self.package_name, auth=self.auth)
        assert response.ok, 'Response was: [{}] {}'.format(
            response.status_code, response.content)
        self.updown_id, version = upload_snap(self.package_name)

    def assert_body_success(self, response):
        body = response.json()
        self.assertTrue(body['success'], 'Response was: [{}] {}'.format(
            response.status_code, body))
        self.assertIn('status_url', body)

    def assert_body_error(self, response):
        body = response.json()
        expected = {
            'errors': {
                'updown_id': ['This field is required.'],
                'name': ['This field is required.']
            },
            'success': False,
            'error_list': [{
                'code': 'missing-field',
                'message': "The 'updown_id' field is required",
                }, {
                'code': 'missing-field',
                'message': "The 'name' field is required",
            }],

        }
        self.assertEqual(body, expected)

    def test_push_snap_successful(self):
        response = push_snap(self.package_name, self.updown_id, auth=self.auth)
        self.assert_success(response)
        self.assert_body_success(response)

    def test_push_snap_invalid_data(self):
        response = push_snap(auth=self.auth)
        self.assert_error(response, content_type='application/json')
        self.assert_body_error(response)

    def test_push_snap_require_authentication(self):
        response = push_snap(self.package_name, 'some-id')
        self.assert_require_auth(response)

    def test_push_snap_require_package_upload_permission(self):
        auth = authenticate_with_macaroon(permissions=['package_access'])
        response = push_snap(self.package_name, self.updown_id, auth=auth)
        self.assert_error(response, content_type='application/json')
        self.assert_forbidden(response, new_style=True)
