import json
import logging
import time
import uuid

import requests

from .helpers import (
    SCA_ROOT_URL,
    APITestCase,
    authenticate_with_macaroon,
    upload_snap,
)
from .test_register_name import register_name
from .test_snap_push import push_snap


RELEASE_SNAP_RETRY_WAIT = 30


def release_snap(name=None, revision=None, channels=None,
                 auth=None, retry=True, status_url=None):
    headers = {
        'Content-Type': 'application/json',
    }
    if auth is not None:
        headers['Authorization'] = auth

    data = {}
    if name is not None:
        data['name'] = name
    if revision is not None:
        data['revision'] = revision
    if channels is not None:
        data['channels'] = channels

    url = '{}/dev/api/snap-release/'.format(SCA_ROOT_URL)
    response = requests.post(url, data=json.dumps(data), headers=headers)

    if retry and response.status_code == 409:
        if status_url is not None:
            response = poll_until_processed(status_url)
        else:
            logging.info(
                'Failed to release: data=%s. Retrying in %s seconds.',
                data, RELEASE_SNAP_RETRY_WAIT)
            time.sleep(RELEASE_SNAP_RETRY_WAIT)
        response = requests.post(url, data=json.dumps(data), headers=headers)

    return response


def poll_until_processed(status_url):
    response = get_snap_status(status_url)
    # TODO: how to avoid looping forever here in some sensible way?
    while not is_processed(response):
        response = get_snap_status(status_url)
    return response


def get_snap_status(status_url):
    response = requests.get(status_url, headers={'Cache-Control': 'no-cache'})
    if not response.ok:
        logging.error('Failed to get snap status. Response was: [%s] %s',
                      response.status_code, response.content)
    return response


def get_snap_status_url(response):
    body = response.json()
    return body.get('status_details_url')


def is_processed(response):
    body = response.json()
    return body.get('processed', False)


def can_release(response):
    body = response.json()
    return body.get('can_release', False)


def get_snap_revision(response):
    body = response.json()
    return body.get('revision')


class SnapReleaseTestCase(APITestCase):

    @classmethod
    def setUpClass(cls):
        super(SnapReleaseTestCase, cls).setUpClass()

        cls.auth = authenticate_with_macaroon()

    def setUp(self):
        super(SnapReleaseTestCase, self).setUp()

        # use same format for package names as snapcraft integration tests
        self.name = 'delete-me-{}'.format(str(uuid.uuid4().int)[:32])
        self.revision = uuid.uuid4().int
        self.channels = ['edge']

    def assert_success_body(self, response, version, revision):
        body = response.json()
        self.assertEqual(body, {
            'opened_channels': ['edge'],
            'success': True,
            'channel_map': [
                {'channel': 'stable', 'info': 'none'},
                {'channel': 'candidate', 'info': 'none'},
                {'channel': 'beta', 'info': 'none'},
                {'revision': revision, 'channel': 'edge', 'info': 'specific',
                 'version': version}
            ]
        })

    def assert_error_body(self, response):
        body = response.json()
        expected = {
            'errors': {
                'channels': ['This field is required.'],
                'name': ['This field is required.'],
                'revision': ['This field is required.'],
            },
            'success': False,
            'error_list': [{
                'code': 'missing-field',
                'message': "The 'channels' field is required",
            }, {
                'code': 'missing-field',
                'message': "The 'name' field is required",
            }, {
                'code': 'missing-field',
                'message': "The 'revision' field is required",
            }],
        }
        self.assertEqual(body, expected)

    def test_release_snap_successful(self):
        response = register_name(self.name, auth=self.auth)
        updown_id, version = upload_snap(self.name)
        response = push_snap(self.name, updown_id, auth=self.auth)
        status_url = get_snap_status_url(response)
        response = poll_until_processed(status_url)
        if not can_release(response):
            self.fail('Unexpected error. Can not release package. '
                      'Status: {}'.format(response.json()))

        revision = get_snap_revision(response)
        response = release_snap(self.name, revision, self.channels,
                                auth=self.auth, status_url=status_url)
        self.assert_success(response)
        self.assert_success_body(response, version, revision)

    def test_release_snap_invalid_data(self):
        response = release_snap(auth=self.auth)
        self.assert_error(response, content_type='application/json')
        self.assert_error_body(response)

    def test_release_snap_require_authentication(self):
        response = release_snap(self.name, self.revision, self.channels)
        self.assert_require_auth(response)

    def test_release_snap_require_package_upload_permission(self):
        auth = authenticate_with_macaroon(permissions=['package_access'])
        response = release_snap(self.name, self.revision, self.channels,
                                auth=auth)
        self.assert_error(response, content_type='application/json')
        self.assert_forbidden(response, new_style=True)
